@@ -103,6 +103,8 @@ prophet_plot_components <- function(
   m, fcst, uncertainty = TRUE, plot_cap = TRUE, weekly_start = 0,
   yearly_start = 0, render_plot = TRUE
 ) {
+  dt <- diff(time_diff(m$history$ds, m$start))
+  min.dt <- min(dt[dt > 0])
   # Plot the trend
   panels <- list(
     plot_forecast_component(m, fcst, 'trend', uncertainty, plot_cap))
@@ -113,7 +115,11 @@ prophet_plot_components <- function(
   }
   # Plot weekly seasonality, if present
   if ("weekly" %in% colnames(fcst)) {
-    panels[[length(panels) + 1]] <- plot_weekly(m, uncertainty, weekly_start)
+    if (min.dt < 1) {
+      panels[[length(panels) + 1]] <- plot_seasonality(m, 'weekly', uncertainty)
+    } else {
+      panels[[length(panels) + 1]] <- plot_weekly(m, uncertainty, weekly_start)
+    }
   }
   # Plot yearly seasonality, if present
   if ("yearly" %in% colnames(fcst)) {
@@ -177,8 +183,17 @@ prophet_plot_components <- function(
 plot_forecast_component <- function(
   m, fcst, name, uncertainty = TRUE, plot_cap = FALSE
 ) {
+
+  wrapped.name <- paste0("`", name, "`")
+  
+  lower.name <- paste0(name, '_lower')
+  lower.name <- paste0("`", lower.name, "`")
+
+  upper.name <- paste0(name, '_upper')
+  upper.name <- paste0("`", upper.name, "`")
+
   gg.comp <- ggplot2::ggplot(
-      fcst, ggplot2::aes_string(x = 'ds', y = name, group = 1)) +
+      fcst, ggplot2::aes_string(x = 'ds', y = wrapped.name, group = 1)) +
     ggplot2::geom_line(color = "#0072B2", na.rm = TRUE)
   if (exists('cap', where = fcst) && plot_cap) {
     gg.comp <- gg.comp + ggplot2::geom_line(
@@ -192,7 +207,7 @@ plot_forecast_component <- function(
     gg.comp <- gg.comp +
       ggplot2::geom_ribbon(
         ggplot2::aes_string(
-          ymin = paste0(name, '_lower'), ymax = paste0(name, '_upper')
+          ymin = lower.name, ymax = upper.name
         ),
         alpha = 0.2,
         fill = "#0072B2",
@@ -339,15 +354,30 @@ plot_seasonality <- function(m, name, uncertainty = TRUE) {
   gg.s <- ggplot2::ggplot(
       seas, ggplot2::aes_string(x = 'ds', y = name, group = 1)) +
     ggplot2::geom_line(color = "#0072B2", na.rm = TRUE)
-  if (period <= 2) {
+
+  date_breaks <- ggplot2::waiver()
+  label <- 'ds'
+  if (name == 'weekly') {
+    fmt.str <- '%a'
+    date_breaks <- '1 day'
+    label <- 'Day of Week'
+  } else if (name == 'daily') {
     fmt.str <- '%T'
+    date_breaks <- '4 hours'
+    label <- 'Hour of day'
+  } else if (period <= 2) {
+    fmt.str <- '%T'
+    label <- 'Hours'
   } else if (period < 14) {
     fmt.str <- '%m/%d %R'
   } else {
     fmt.str <- '%m/%d'
   }
   gg.s <- gg.s +
-    ggplot2::scale_x_datetime(labels = scales::date_format(fmt.str))
+    ggplot2::scale_x_datetime(
+      labels = scales::date_format(fmt.str), date_breaks = date_breaks
+    ) +
+    ggplot2::xlab(label)
   if (uncertainty && m$uncertainty.samples) {
     gg.s <- gg.s +
     ggplot2::geom_ribbon(
@@ -405,7 +435,7 @@ add_changepoints_to_plot <- function(m, threshold = 0.01, cp_color = "red",
 #' @param uncertainty Optional boolean indicating if the uncertainty interval for yhat
 #'  should be plotted, which will only be done if x$uncertainty.samples > 0. Must be
 #'  present in fcst as yhat_lower and yhat_upper.
-#' @param ... additional arguments passed to dygraph::dygraph
+#' @param ... additional arguments passed to dygraphs::dygraph
 #' @importFrom dplyr "%>%"
 #' @return A dygraph plot.
 #'
@@ -440,7 +470,7 @@ dyplot.prophet <- function(x, fcst, uncertainty=TRUE,
     forecastCols <- c('yhat')
   }
   # convert to xts for easier date handling by dygraph
-  dfTS <- xts::xts(df %>% dplyr::select_(.dots=colsToKeep), order.by = df$ds)
+  dfTS <- xts::xts(df %>% dplyr::select(.dots=colsToKeep), order.by = df$ds)
 
   # base plot
   dyBase <- dygraphs::dygraph(dfTS, ...)
