@@ -16,54 +16,56 @@ globalVariables(c(
 #' Prophet forecaster.
 #'
 #' @param df (optional) Dataframe containing the history. Must have columns ds
-#'  (date type) and y, the time series. If growth is logistic, then df must
-#'  also have a column cap that specifies the capacity at each ds. If not
-#'  provided, then the model object will be instantiated but not fit; use
-#'  fit.prophet(m, df) to fit the model.
-#' @param growth String 'linear' or 'logistic' to specify a linear or logistic
-#'  trend.
+#'   (date type) and y, the time series. If growth is logistic, then df must
+#'   also have a column cap that specifies the capacity at each ds. If not
+#'   provided, then the model object will be instantiated but not fit; use
+#'   fit.prophet(m, df) to fit the model.
+#' @param growth String 'linear', 'logistic', or 'flat' to specify a linear,
+#'   logistic or flat trend.
 #' @param changepoints Vector of dates at which to include potential
-#'  changepoints. If not specified, potential changepoints are selected
-#'  automatically.
+#'   changepoints. If not specified, potential changepoints are selected
+#'   automatically.
 #' @param n.changepoints Number of potential changepoints to include. Not used
-#'  if input `changepoints` is supplied. If `changepoints` is not supplied,
-#'  then n.changepoints potential changepoints are selected uniformly from the
-#'  first `changepoint.range` proportion of df$ds.
+#'   if input `changepoints` is supplied. If `changepoints` is not supplied,
+#'   then n.changepoints potential changepoints are selected uniformly from the
+#'   first `changepoint.range` proportion of df$ds.
 #' @param changepoint.range Proportion of history in which trend changepoints
-#'  will be estimated. Defaults to 0.8 for the first 80%. Not used if
-#'  `changepoints` is specified.
-#' @param yearly.seasonality Fit yearly seasonality. Can be 'auto', TRUE,
-#'  FALSE, or a number of Fourier terms to generate.
-#' @param weekly.seasonality Fit weekly seasonality. Can be 'auto', TRUE,
-#'  FALSE, or a number of Fourier terms to generate.
-#' @param daily.seasonality Fit daily seasonality. Can be 'auto', TRUE,
-#' FALSE, or a number of Fourier terms to generate.
+#'   will be estimated. Defaults to 0.8 for the first 80%. Not used if
+#'   `changepoints` is specified.
+#' @param yearly.seasonality Fit yearly seasonality. Can be 'auto', TRUE, FALSE,
+#'   or a number of Fourier terms to generate.
+#' @param weekly.seasonality Fit weekly seasonality. Can be 'auto', TRUE, FALSE,
+#'   or a number of Fourier terms to generate.
+#' @param daily.seasonality Fit daily seasonality. Can be 'auto', TRUE, FALSE,
+#'   or a number of Fourier terms to generate.
 #' @param holidays data frame with columns holiday (character) and ds (date
-#'  type)and optionally columns lower_window and upper_window which specify a
-#'  range of days around the date to be included as holidays. lower_window=-2
-#'  will include 2 days prior to the date as holidays. Also optionally can have
-#'  a column prior_scale specifying the prior scale for each holiday.
+#'   type)and optionally columns lower_window and upper_window which specify a
+#'   range of days around the date to be included as holidays. lower_window=-2
+#'   will include 2 days prior to the date as holidays. Also optionally can have
+#'   a column prior_scale specifying the prior scale for each holiday.
 #' @param seasonality.mode 'additive' (default) or 'multiplicative'.
 #' @param seasonality.prior.scale Parameter modulating the strength of the
-#'  seasonality model. Larger values allow the model to fit larger seasonal
-#'  fluctuations, smaller values dampen the seasonality. Can be specified for
-#'  individual seasonalities using add_seasonality.
+#'   seasonality model. Larger values allow the model to fit larger seasonal
+#'   fluctuations, smaller values dampen the seasonality. Can be specified for
+#'   individual seasonalities using add_seasonality.
 #' @param holidays.prior.scale Parameter modulating the strength of the holiday
-#'  components model, unless overridden in the holidays input.
+#'   components model, unless overridden in the holidays input.
 #' @param changepoint.prior.scale Parameter modulating the flexibility of the
-#'  automatic changepoint selection. Large values will allow many changepoints,
-#'  small values will allow few changepoints.
+#'   automatic changepoint selection. Large values will allow many changepoints,
+#'   small values will allow few changepoints.
 #' @param mcmc.samples Integer, if greater than 0, will do full Bayesian
-#'  inference with the specified number of MCMC samples. If 0, will do MAP
-#'  estimation.
+#'   inference with the specified number of MCMC samples. If 0, will do MAP
+#'   estimation.
 #' @param interval.width Numeric, width of the uncertainty intervals provided
-#'  for the forecast. If mcmc.samples=0, this will be only the uncertainty
-#'  in the trend using the MAP estimate of the extrapolated generative model.
-#'  If mcmc.samples>0, this will be integrated over all model parameters,
-#'  which will include uncertainty in seasonality.
+#'   for the forecast. If mcmc.samples=0, this will be only the uncertainty in
+#'   the trend using the MAP estimate of the extrapolated generative model. If
+#'   mcmc.samples>0, this will be integrated over all model parameters, which
+#'   will include uncertainty in seasonality.
 #' @param uncertainty.samples Number of simulated draws used to estimate
-#'  uncertainty intervals. Settings this value to 0 or False will disable
-#'  uncertainty estimation and speed up the calculation.
+#'   uncertainty intervals. Settings this value to 0 or False will disable
+#'   uncertainty estimation and speed up the calculation.
+#' @param backend Whether to use the "rstan" or "cmdstanr" backend to fit the
+#'   model. If not provided, uses the R_STAN_BACKEND environment variable.
 #' @param fit Boolean, if FALSE the model is initialized but not fit.
 #' @param ... Additional arguments, passed to \code{\link{fit.prophet}}
 #'
@@ -79,6 +81,7 @@ globalVariables(c(
 #' @export
 #' @importFrom dplyr "%>%"
 #' @import Rcpp
+#' @rawNamespace import(RcppParallel, except = LdFlags)
 #' @import rlang
 #' @useDynLib prophet, .registration = TRUE
 prophet <- function(df = NULL,
@@ -98,12 +101,15 @@ prophet <- function(df = NULL,
                     interval.width = 0.80,
                     uncertainty.samples = 1000,
                     fit = TRUE,
+                    backend = NULL,
                     ...
 ) {
   if (!is.null(changepoints)) {
     n.changepoints <- length(changepoints)
   }
 
+  if (is.null(backend)) backend <- get_stan_backend()
+
   m <- list(
     growth = growth,
     changepoints = changepoints,
@@ -120,6 +126,7 @@ prophet <- function(df = NULL,
     mcmc.samples = mcmc.samples,
     interval.width = interval.width,
     uncertainty.samples = uncertainty.samples,
+    backend = backend,
     specified.changepoints = !is.null(changepoints),
     start = NULL,  # This and following attributes are set during fitting
     y.scale = NULL,
@@ -154,8 +161,8 @@ prophet <- function(df = NULL,
 #'
 #' @keywords internal
 validate_inputs <- function(m) {
-  if (!(m$growth %in% c('linear', 'logistic'))) {
-    stop("Parameter 'growth' should be 'linear' or 'logistic'.")
+  if (!(m$growth %in% c('linear', 'logistic', 'flat'))) {
+    stop("Parameter 'growth' should be 'linear', 'logistic', or 'flat'.")
   }
   if ((m$changepoint.range < 0) | (m$changepoint.range > 1)) {
     stop("Parameter 'changepoint.range' must be in [0, 1]")
@@ -168,6 +175,9 @@ validate_inputs <- function(m) {
       stop('Holidays dataframe must have ds field.')
     }
     m$holidays$ds <- as.Date(m$holidays$ds)
+    if (any(is.na(m$holidays$ds)) | any(is.na(m$holidays$holiday))) {
+      stop('Found NA in the holidays dataframe.')
+    }
     has.lower <- exists('lower_window', where = m$holidays)
     has.upper <- exists('upper_window', where = m$holidays)
     if (has.lower + has.upper == 1) {
@@ -239,15 +249,15 @@ validate_column_name <- function(
 
 #' Convert date vector
 #'
-#' Convert the date to POSIXct object
+#' Convert the date to POSIXct object. Timezones are stripped and replaced
+#' with GMT.
 #'
-#' @param ds Date vector, can be consisted of characters
-#' @param tz string time zone
+#' @param ds Date vector
 #'
 #' @return vector of POSIXct object converted from date
 #'
 #' @keywords internal
-set_date <- function(ds = NULL, tz = "GMT") {
+set_date <- function(ds) {
   if (length(ds) == 0) {
     return(NULL)
   }
@@ -256,12 +266,20 @@ set_date <- function(ds = NULL, tz = "GMT") {
     ds <- as.character(ds)
   }
 
-  if (min(nchar(ds), na.rm=TRUE) < 12) {
-    ds <- as.POSIXct(ds, format = "%Y-%m-%d", tz = tz)
-  } else {
-    ds <- as.POSIXct(ds, format = "%Y-%m-%d %H:%M:%S", tz = tz)
+  # If a datetime, strip timezone and replace with GMT.
+  if (lubridate::is.instant(ds)) {
+    ds <- as.POSIXct(lubridate::force_tz(ds, "GMT"), tz = "GMT")
   }
-  attr(ds, "tzone") <- tz
+  else {
+    # Assume it can be coerced into POSIXct
+    if (min(nchar(ds), na.rm=TRUE) < 12) {
+        ds <- as.POSIXct(ds, format = "%Y-%m-%d", tz = "GMT")
+    } else {
+        ds <- as.POSIXct(ds, format = "%Y-%m-%d %H:%M:%S", tz = "GMT")
+    }
+  }
+
+  attr(ds, "tzone") <- "GMT"
   return(ds)
 }
 
@@ -282,7 +300,7 @@ time_diff <- function(ds1, ds2, units = "days") {
 
 #' Prepare dataframe for fitting or predicting.
 #'
-#' Adds a time index and scales y. Creates auxillary columns 't', 't_ix',
+#' Adds a time index and scales y. Creates auxiliary columns 't', 't_ix',
 #' 'y_scaled', and 'cap_scaled'. These columns are used during both fitting
 #' and predicting.
 #'
@@ -529,7 +547,9 @@ construct_holiday_dataframe <- function(m, dates) {
     holidays.to.add <- data.frame(
       holiday=setdiff(m$train.holiday.names, all.holidays$holiday)
     )
-    all.holidays <- suppressWarnings(dplyr::bind_rows(all.holidays, holidays.to.add))
+    if (nrow(holidays.to.add) > 0) {
+      all.holidays <- suppressWarnings(dplyr::bind_rows(all.holidays, holidays.to.add))
+    }
   }
   return(all.holidays)
 }
@@ -798,7 +818,7 @@ add_country_holidays <- function(m, country_name) {
 #'
 #' @return List with items
 #'  seasonal.features: Dataframe with regressor features,
-#'  prior.scales: Array of prior scales for each colum of the features
+#'  prior.scales: Array of prior scales for each column of the features
 #'    dataframe.
 #'  component.cols: Dataframe with indicators for which regression components
 #'    correspond to which columns.
@@ -983,9 +1003,9 @@ parse_seasonality_args <- function(m, name, arg, auto.disable, default.order) {
     } else {
       fourier.order <- default.order
     }
-  } else if (arg == TRUE) {
+  } else if (isTRUE(arg)) {
     fourier.order <- default.order
-  } else if (arg == FALSE) {
+  } else if (isFALSE(arg)) {
     fourier.order <- 0
   } else {
     fourier.order <- arg
@@ -1053,7 +1073,28 @@ set_auto_seasonalities <- function(m) {
   return(m)
 }
 
-#' Initialize linear growth.
+#' Initialize flat growth.
+#'
+#' Provides a strong initialization for flat growth by setting the
+#' growth to 0 and calculates the offset parameter that pass the
+#' function through the mean of the the y_scaled values.
+#'
+#' @param df Data frame with columns ds (date), y_scaled (scaled time series),
+#'  and t (scaled time).
+#'
+#' @return A vector (k, m) with the rate (k) and offset (m) of the flat
+#'  growth function.
+#'
+#' @keywords internal
+flat_growth_init <- function(df) {
+  # Initialize the rate
+  k <- 0
+  # And the offset
+  m <- mean(df$y_scaled)
+  return(c(k, m))
+}
+
+#' Initialize constant growth.
 #'
 #' Provides a strong initialization for linear growth by calculating the
 #' growth and offset parameters that pass the function through the first and
@@ -1148,7 +1189,7 @@ fit.prophet <- function(m, df, ...) {
   if (nrow(history) < 2) {
     stop("Dataframe has less than 2 non-NA rows.")
   }
-  m$history.dates <- sort(set_date(df$ds))
+  m$history.dates <- sort(set_date(unique(df$ds)))
 
   out <- setup_dataframe(m, history, initialize_scales = TRUE)
   history <- out$df
@@ -1177,7 +1218,7 @@ fit.prophet <- function(m, df, ...) {
     X = as.matrix(seasonal.features),
     sigmas = array(prior.scales),
     tau = m$changepoint.prior.scale,
-    trend_indicator = as.numeric(m$growth == 'logistic'),
+    trend_indicator = switch(m$growth, 'linear'=0, 'logistic'=1, 'flat'=2),
     s_a = array(component.cols$additive_terms),
     s_m = array(component.cols$multiplicative_terms)
   )
@@ -1186,16 +1227,15 @@ fit.prophet <- function(m, df, ...) {
   if (m$growth == 'linear') {
     dat$cap <- rep(0, nrow(history))  # Unused inside Stan
     kinit <- linear_growth_init(history)
-  } else {
+  } else if (m$growth == 'flat') {
+    dat$cap <- rep(0, nrow(history)) # Unused inside Stan
+    kinit <- flat_growth_init(history)
+  } else if (m$growth == 'logistic') {
     dat$cap <- history$cap_scaled  # Add capacities to the Stan data
     kinit <- logistic_growth_init(history)
   }
 
-  if (exists(".prophet.stan.model", where = prophet_model_env)) {
-    model <- get('.prophet.stan.model', envir = prophet_model_env)
-  } else {
-    model <- stanmodels$prophet
-  }
+  model <- .load_model(m$backend)
 
   stan_init <- function() {
     list(k = kinit[1],
@@ -1206,44 +1246,24 @@ fit.prophet <- function(m, df, ...) {
     )
   }
 
-  if (min(history$y) == max(history$y)) {
+  if (min(history$y) == max(history$y) &
+        (m$growth %in% c('linear', 'flat'))) {
     # Nothing to fit.
     m$params <- stan_init()
     m$params$sigma_obs <- 0.
     n.iteration <- 1.
-  } else if (m$mcmc.samples > 0) {
-    args <- list(
-      object = model,
-      data = dat,
-      init = stan_init,
-      iter = m$mcmc.samples
-    )
-    args <- utils::modifyList(args, list(...))
-    m$stan.fit <- do.call(rstan::sampling, args)
-    m$params <- rstan::extract(m$stan.fit)
-    n.iteration <- length(m$params$k)
   } else {
-    args <- list(
-      object = model,
-      data = dat,
-      init = stan_init,
-      algorithm = if(dat$T < 100) {'Newton'} else {'LBFGS'},
-      iter = 1e4,
-      as_vector = FALSE
-    )
-    args <- utils::modifyList(args, list(...))
-    m$stan.fit <- do.call(rstan::optimizing, args)
-    if (m$stan.fit$return_code != 0) {
-      message(
-        'Optimization terminated abnormally. Falling back to Newton optimizer.'
-      )
-      args$algorithm = 'Newton'
-      m$stan.fit <- do.call(rstan::optimizing, args)
+    if (m$mcmc.samples > 0) {
+      args <- .stan_args(model, dat, stan_init, m$backend, type = "mcmc", m$mcmc.samples, ...)
+      model_output <- .sampling(args, m$backend)
+    } else {
+      args <- .stan_args(model, dat, stan_init, m$backend, type = "optimize", ...)
+      model_output <- .fit(args, m$backend)
     }
-    m$params <- m$stan.fit$par
-    n.iteration <- 1
+    m$stan.fit <- model_output$stan_fit
+    m$params <- model_output$params
+    n.iteration <- model_output$n_iteration
   }
-
   # Cast the parameters to have consistent form, whether full bayes or MAP
   for (name in c('delta', 'beta')){
     m$params[[name]] <- matrix(m$params[[name]], nrow = n.iteration)
@@ -1318,6 +1338,19 @@ predict.prophet <- function(object, df = NULL, ...) {
   return(df)
 }
 
+#' Evaluate the flat trend function.
+#'
+#' @param t Vector of times on which the function is evaluated.
+#' @param m Float initial offset.
+#'
+#' @return Vector y(t).
+#'
+#' @keywords internal
+flat_trend <- function(t, m) {
+  y <- rep(m, length(t))
+  return(y)
+}
+
 #' Evaluate the piecewise linear function.
 #'
 #' @param t Vector of times on which the function is evaluated.
@@ -1392,7 +1425,9 @@ predict_trend <- function(model, df) {
   t <- df$t
   if (model$growth == 'linear') {
     trend <- piecewise_linear(t, deltas, k, param.m, model$changepoints.t)
-  } else {
+  } else if (model$growth == 'flat') {
+     trend <- flat_trend(t, param.m)
+  } else if (model$growth == 'logistic') {
     cap <- df$cap_scaled
     trend <- piecewise_logistic(
       t, cap, deltas, k, param.m, model$changepoints.t)
@@ -1592,7 +1627,9 @@ sample_predictive_trend <- function(model, df, iteration) {
   # Get the corresponding trend
   if (model$growth == 'linear') {
     trend <- piecewise_linear(t, deltas, k, param.m, changepoint.ts)
-  } else {
+  } else if (model$growth == 'flat') {
+    trend <- flat_trend(t, param.m)
+  } else if (model$growth == 'logistic') {
     cap <- df$cap_scaled
     trend <- piecewise_logistic(t, cap, deltas, k, param.m, changepoint.ts)
   }
@@ -1613,7 +1650,7 @@ sample_predictive_trend <- function(model, df, iteration) {
 #' @export
 make_future_dataframe <- function(m, periods, freq = 'day',
                                   include_history = TRUE) {
-  # For backwards compatability with previous zoo date type,
+  # For backwards compatibility with previous zoo date type,
   if (freq == 'm') {
     freq <- 'month'
   }
