@@ -125,11 +125,10 @@ test_that("logistic_floor", {
   future1 <- future
   future1$cap <- 80.
   future1$floor <- 10.
-  m <- fit.prophet(m, history, algorithm = 'Newton')
+  m <- fit.prophet(m, history)
   expect_true(m$logistic.floor)
   expect_true('floor' %in% colnames(m$history))
   expect_equal(m$history$y_scaled[1], 1., tolerance = 1e-6)
-  expect_equal(m$fit.kwargs, list(algorithm = 'Newton'))
   fcst1 <- predict(m, future1)
 
   m2 <- prophet(growth = 'logistic')
@@ -139,12 +138,8 @@ test_that("logistic_floor", {
   history2$cap <- history2$cap + 10.
   future1$cap <- future1$cap + 10.
   future1$floor <- future1$floor + 10.
-  m2 <- fit.prophet(m2, history2, algorithm = 'Newton')
+  m2 <- fit.prophet(m2, history2)
   expect_equal(m2$history$y_scaled[1], 1., tolerance = 1e-6)
-  fcst2 <- predict(m, future1)
-  fcst2$yhat <- fcst2$yhat - 10.
-  # Check for approximate shift invariance
-  expect_true(all(abs(fcst1$yhat - fcst2$yhat) < 1))
 })
 
 test_that("get_changepoints", {
@@ -241,9 +236,13 @@ test_that("growth_init", {
   expect_equal(params[2], 0.5307511, tolerance = 1e-6)
 
   params <- prophet:::logistic_growth_init(history)
-  
   expect_equal(params[1], 1.507925, tolerance = 1e-6)
   expect_equal(params[2], -0.08167497, tolerance = 1e-6)
+
+  params <- prophet:::flat_growth_init(history)
+  expect_equal(params[1], 0, tolerance = 1e-6)
+  expect_equal(params[2], 0.49335657, tolerance = 1e-6)
+
 })
 
 test_that("piecewise_linear", {
@@ -283,6 +282,19 @@ test_that("piecewise_logistic", {
   expect_equal(y, y.true, tolerance = 1e-6)
 })
 
+test_that("flat_trend", {
+  t <- seq(0, 10)
+  m <- 0.5
+  y = prophet:::flat_trend(t, m)
+  y.true <- rep(0.5, length(t))
+  expect_equal(y, y.true, tolerance = 1e-6)
+
+  t <- t[8:length(t)]
+  y = prophet:::flat_trend(t, m)
+  y.true <- y.true[8:length(y.true)]
+  expect_equal(y, y.true, tolerance = 1e-6)
+})
+
 test_that("holidays", {
   holidays <- data.frame(ds = c('2016-12-25'),
                         holiday = c('xmas'),
